@@ -1223,6 +1223,14 @@ class _DocCreatorVisitor:
 
         return _DocCreatorVisitor._encoding_map[s]
 
+    @staticmethod
+    def _is_power_of_two(i):
+        return ((i & (i - 1)) == 0) and i > 0
+
+    @staticmethod
+    def _is_size_valid(sz):
+        return sz >= 1
+
     def _reset_state(self):
         self._objs = []
         self._scope_stores = []
@@ -1297,7 +1305,8 @@ class _DocCreatorVisitor:
 
     def visit_Top(self, node):
         self._reset_state()
-        self._doc = self._visit_scope(node, pytsdl.tsdl.Doc())
+        self._doc = pytsdl.tsdl.Doc()
+        self._visit_scope(node, self._doc)
 
         # ensure at least one clock, at least one stream
         if not self._doc.clocks:
@@ -1534,6 +1543,38 @@ class _DocCreatorVisitor:
         if integer.size is None:
             raise ParseError('integer missing size')
 
+        if not _DocCreatorVisitor._is_size_valid(integer.size):
+            raise ParseError('wrong integer size: {}'.format(integer.size))
+
+        if integer.align is None:
+            if integer.size % 8 == 0:
+                integer.align = 8
+            else:
+                integer.align = 1
+
+        if not _DocCreatorVisitor._is_power_of_two(integer.align):
+            raise ParseError('wrong integer alignment: {}'.format(integer.align))
+
+        if integer.base not in [2, 8, 10, 16]:
+            raise ParseError('wrong integer base: {}'.format(integer.base))
+
+        # verify clock mapping
+        if integer.map is not None:
+            m = '.'.join(integer.map)
+
+            if len(integer.map) != 3:
+                raise ParseError('wrong integer map: {}'.format(m))
+
+            if integer.map[0] != 'clock':
+                raise ParseError('integer map does not start with "clock": {}"'.format(m))
+
+            if integer.map[2] != 'value':
+                raise ParseError('integer map does not end with "value": {}"'.format(m))
+
+            # search clock
+            if integer.map[1] not in self._doc.clocks:
+                raise ParseError('integer does not map to an existing clock: {}"'.format(m))
+
         return integer
 
     def _floating_point_to_obj(self, t):
@@ -1550,6 +1591,21 @@ class _DocCreatorVisitor:
         if floating_point.mant_dig is None:
             raise ParseError('floating point missing mantissa digits')
 
+        if not _DocCreatorVisitor._is_size_valid(floating_point.exp_dig):
+            raise ParseError('wrong floating point exponent size: {}'.format(floating_point.exp_dig))
+
+        if not _DocCreatorVisitor._is_size_valid(floating_point.mant_dig):
+            raise ParseError('wrong floating point mantissa size: {}'.format(floating_point.mant_dig))
+
+        if floating_point.align is None:
+            if (floating_point.exp_dig + floating_point.mant_dig) % 8 == 0:
+                floating_point.align = 8
+            else:
+                floating_point.align = 1
+
+        if not _DocCreatorVisitor._is_power_of_two(floating_point.align):
+            raise ParseError('wrong floating point alignment: {}'.format(floating_point.align))
+
         return floating_point
 
     def _string_to_obj(self, t):
@@ -1609,6 +1665,11 @@ class _DocCreatorVisitor:
             obj.length = _DocCreatorVisitor._decode_unary(subscript.value.expr)
         elif type(subscript.value.expr) is ConstNumber:
             obj = pytsdl.tsdl.Array()
+            length = subscript.value.expr.value
+
+            if length < 1:
+                raise ParseError('wrong array length: {}'.format(length))
+
             obj.length = subscript.value.expr.value
         else:
             raise ParseError('invalid subscript type: {}'.format(subscript.value.expr))
@@ -1660,6 +1721,9 @@ class _DocCreatorVisitor:
         if t.align:
             struct.align = t.align.value.value
 
+            if not _DocCreatorVisitor._is_power_of_two(struct.align):
+                raise ParseError('wrong structure alignment: {}'.format(struct.align))
+
         # store this struct if it's named
         if t.name is not None:
             self._store_struct(t.name.value, struct)
@@ -1751,9 +1815,14 @@ class Parser:
 
         return ast
 
-    def parse(self, tsdl):
-        ast = self.get_ast(tsdl)
+    @staticmethod
+    def _validate_magic(tsdl):
+        if not tsdl.startswith('/* CTF 1.8'):
+            raise ParseError('TSDL document must start with exactly "/* CTF 1.8"')
 
+    def parse(self, tsdl):
+        Parser._validate_magic(tsdl)
+        ast = self.get_ast(tsdl)
         visitor = _DocCreatorVisitor()
         ast.accept(visitor)
 
